@@ -59,11 +59,10 @@ impl CDFileStats {
         let total = coverage.len();
         let mut lines: Vec<i64> = vec![-1; last_line];
         for (line_num, line_count) in coverage.iter() {
-            let line_count = *line_count;
-            unsafe {
-                *lines.get_unchecked_mut((*line_num - 1) as usize) = line_count as i64;
+            if let Some(line) = lines.get_mut((*line_num - 1) as usize) {
+                *line = *line_count as i64;
+                covered += (*line_count > 0) as usize;
             }
-            covered += (line_count > 0) as usize;
         }
         (total, covered, lines)
     }
