@@ -1,4 +1,5 @@
 use chrono::{DateTime, Utc};
+use log::warn;
 use serde::{Deserialize, Serialize};
 use serde_json::value::{from_value, to_value, Value};
 use std::borrow::Cow;
@@ -8,7 +9,7 @@ use std::fs::{self, File};
 use std::io::{BufReader, Read, Write};
 use std::path::{Path, PathBuf};
 use std::sync::{Arc, Mutex};
-use tera::try_get_value;
+use tera::{try_get_value, Context, Tera};
 
 use crate::defs::*;
 
@@ -24,6 +25,12 @@ impl HtmlStats {
     }
 }
 
+#[derive(Debug, clap::ValueEnum, Clone, Copy)]
+pub enum HtmlResources {
+    Bundled,
+    Cdn,
+}
+
 #[derive(Clone, Debug)]
 pub struct Config {
     hi_limit: f64,
@@ -32,11 +39,20 @@ pub struct Config {
     fn_med_limit: f64,
     branch_hi_limit: f64,
     branch_med_limit: f64,
-    date: DateTime<Utc>,
+    date: Option<DateTime<Utc>>,
+    branch_enabled: bool,
+    precision: usize,
+    html_resources: HtmlResources,
 }
 
 impl Config {
-    fn new(cfg: &ConfigFile) -> Config {
+    fn new(
+        cfg: &ConfigFile,
+        branch_enabled: bool,
+        precision: usize,
+        no_date: bool,
+        html_resources: HtmlResources,
+    ) -> Config {
         Config {
             hi_limit: cfg.hi_limit.unwrap_or(90.),
             med_limit: cfg.med_limit.unwrap_or(75.),
@@ -44,7 +60,10 @@ impl Config {
             fn_med_limit: cfg.fn_med_limit.unwrap_or(75.),
             branch_hi_limit: cfg.branch_hi_limit.unwrap_or(90.),
             branch_med_limit: cfg.branch_med_limit.unwrap_or(75.),
-            date: Utc::now(),
+            date: if no_date { None } else { Some(Utc::now()) },
+            branch_enabled,
+            precision,
+            html_resources,
         }
     }
 }
@@ -72,7 +91,8 @@ impl ConfigFile {
     }
 }
 
-static BULMA_VERSION: &str = "0.9.1";
+static BULMA_CDN_VERSION: &str = "0.9.1";
+static BULMA_BUNDLED: &[u8] = include_bytes!("bulma/bulma-0.9.1.min.css");
 
 fn load_template(path: &str) -> String {
     fs::read_to_string(path).unwrap()
@@ -118,9 +138,21 @@ fn get_templates(user_templates: &Option<HashMap<String, String>>) -> HashMap<St
     result
 }
 
-pub fn get_config(output_config_file: Option<&Path>) -> (Tera, Config) {
+pub fn get_config(
+    output_config_file: Option<&Path>,
+    branch_enabled: bool,
+    precision: usize,
+    no_date: bool,
+    html_resources: HtmlResources,
+) -> (Tera, Config) {
     let user_conf = ConfigFile::load(output_config_file);
-    let conf = Config::new(&user_conf);
+    let conf = Config::new(
+        &user_conf,
+        branch_enabled,
+        precision,
+        no_date,
+        html_resources,
+    );
 
     let mut tera = Tera::default();
 
@@ -235,11 +267,16 @@ fn get_base(rel_path: &Path) -> String {
 fn get_dirs_result(global: Arc<Mutex<HtmlGlobalStats>>, rel_path: &Path, stats: &HtmlStats) {
     let parent = rel_path.parent().unwrap().to_str().unwrap().to_string();
     let file_name = rel_path.file_name().unwrap().to_str().unwrap().to_string();
+    let mut global = global.lock().unwrap();
     let fs = HtmlFileStats {
         stats: stats.clone(),
+        abs_prefix: global
+            .abs_prefix
+            .as_ref()
+            .map(|p| Path::new(p).join(&parent)),
     };
-    let mut global = global.lock().unwrap();
     global.stats.add(stats);
+    let p = global.abs_prefix.clone();
     let entry = global.dirs.entry(parent);
     match entry {
         btree_map::Entry::Occupied(ds) => {
@@ -253,67 +290,65 @@ fn get_dirs_result(global: Arc<Mutex<HtmlGlobalStats>>, rel_path: &Path, stats:
             v.insert(HtmlDirStats {
                 files,
                 stats: stats.clone(),
+                abs_prefix: p,
             });
         }
     };
 }
 
-use tera::{Context, Tera};
-
-fn make_context() -> Context {
+fn make_context(conf: &Config, path_to_root: &str) -> Context {
     let mut ctx = Context::new();
-    let ver = std::env::var("BULMA_VERSION").map_or(BULMA_VERSION.into(), |v| v);
-    ctx.insert("bulma_version", &ver);
+
+    let overridden_bulma = std::env::var("BULMA_VERSION").ok();
+    let bulma_url = match conf.html_resources {
+        HtmlResources::Bundled => {
+            if overridden_bulma.is_some() {
+                warn!("BULMA_VERSION env var is incompatible with --html-resources=bundled");
+            }
+            format!("{}/bulma.min.css", path_to_root.trim_end_matches('/'))
+        }
+        HtmlResources::Cdn => {
+            let version = overridden_bulma.as_deref().unwrap_or(BULMA_CDN_VERSION);
+            format!("https://cdn.jsdelivr.net/npm/bulma@{version}/css/bulma.min.css")
+        }
+    };
+    ctx.insert("bulma_url", &bulma_url);
+
+    ctx.insert("date", &conf.date);
+    ctx.insert("precision", &conf.precision);
+    ctx.insert("branch_enabled", &conf.branch_enabled);
 
     ctx
 }
 
-pub fn gen_index(
-    tera: &Tera,
-    global: &HtmlGlobalStats,
-    conf: &Config,
-    output: &Path,
-    branch_enabled: bool,
-    precision: usize,
-) {
+pub fn gen_index(tera: &Tera, global: &HtmlGlobalStats, conf: &Config, output: &Path) {
     let output_file = output.join("index.html");
     create_parent(&output_file);
     let mut output_stream = match File::create(&output_file) {
         Err(_) => {
-            eprintln!("Cannot create file {:?}", output_file);
+            eprintln!("Cannot create file {output_file:?}");
             return;
         }
         Ok(f) => f,
     };
 
-    let mut ctx = make_context();
+    let mut ctx = make_context(conf, ".");
     let empty: &[&str] = &[];
-    ctx.insert("date", &conf.date);
     ctx.insert("current", "top_level");
     ctx.insert("parents", empty);
     ctx.insert("stats", &global.stats);
-    ctx.insert("precision", &precision);
     ctx.insert("items", &global.dirs);
     ctx.insert("kind", "Directory");
-    ctx.insert("branch_enabled", &branch_enabled);
 
     let out = tera.render("index.html", &ctx).unwrap();
 
     if output_stream.write_all(out.as_bytes()).is_err() {
-        eprintln!("Cannot write the file {:?}", output_file);
+        eprintln!("Cannot write the file {output_file:?}");
         return;
     }
 
     for (dir_name, dir_stats) in global.dirs.iter() {
-        gen_dir_index(
-            tera,
-            dir_name,
-            dir_stats,
-            conf,
-            output,
-            branch_enabled,
-            precision,
-        );
+        gen_dir_index(tera, dir_name, dir_stats, conf, output);
     }
 }
 
@@ -323,8 +358,6 @@ pub fn gen_dir_index(
     dir_stats: &HtmlDirStats,
     conf: &Config,
     output: &Path,
-    branch_enabled: bool,
-    precision: usize,
 ) {
     let index = Path::new(dir_name).join("index.html");
     let layers = index.components().count() - 1;
@@ -333,27 +366,27 @@ pub fn gen_dir_index(
     create_parent(&output_file);
     let mut output = match File::create(&output_file) {
         Err(_) => {
-            eprintln!("Cannot create file {:?}", output_file);
+            eprintln!("Cannot create file {output_file:?}");
             return;
         }
         Ok(f) => f,
     };
 
-    let mut ctx = make_context();
-    ctx.insert("date", &conf.date);
-    ctx.insert("bulma_version", BULMA_VERSION);
+    let mut ctx = make_context(conf, &"../".repeat(layers));
     ctx.insert("current", dir_name);
-    ctx.insert("parents", &[(prefix, "top_level")]);
+    let parent_link = match &dir_stats.abs_prefix {
+        Some(p) => p.join(PathBuf::from("index.html")),
+        None => prefix.into(),
+    };
+    ctx.insert("parents", &[(parent_link, "top_level")]);
     ctx.insert("stats", &dir_stats.stats);
     ctx.insert("items", &dir_stats.files);
     ctx.insert("kind", "File");
-    ctx.insert("branch_enabled", &branch_enabled);
-    ctx.insert("precision", &precision);
 
     let out = tera.render("index.html", &ctx).unwrap();
 
     if output.write_all(out.as_bytes()).is_err() {
-        eprintln!("Cannot write the file {:?}", output_file);
+        eprintln!("Cannot write the file {output_file:?}");
     }
 }
 
@@ -365,8 +398,7 @@ fn gen_html(
     output: &Path,
     rel_path: &Path,
     global: Arc<Mutex<HtmlGlobalStats>>,
-    branch_enabled: bool,
-    precision: usize,
+    abs_link_prefix: &Option<String>,
 ) {
     if !rel_path.is_relative() {
         return;
@@ -387,7 +419,7 @@ fn gen_html(
     create_parent(&output_file);
     let mut output = match File::create(&output_file) {
         Err(_) => {
-            eprintln!("Cannot create file {:?}", output_file);
+            eprintln!("Cannot create file {output_file:?}");
             return;
         }
         Ok(f) => f,
@@ -398,20 +430,32 @@ fn gen_html(
     let mut index_url = base_url;
     index_url.push_str("index.html");
 
-    let mut ctx = make_context();
-    ctx.insert("date", &conf.date);
-    ctx.insert("bulma_version", BULMA_VERSION);
+    let mut ctx = make_context(conf, &get_base(rel_path));
     ctx.insert("current", filename);
+
+    let (top_level_link, parent_link) = match abs_link_prefix {
+        Some(prefix) => {
+            let prefix_buf = PathBuf::from(prefix);
+            let mut top_level = prefix_buf.clone();
+            top_level.push("index.html");
+            let parent_buf = PathBuf::from(parent.clone());
+            let mut abs_parent = prefix_buf.join(parent_buf);
+            abs_parent.push(PathBuf::from("index.html"));
+            (
+                top_level.display().to_string(),
+                abs_parent.display().to_string(),
+            )
+        }
+        None => (index_url.to_string(), "./index.html".to_string()),
+    };
     ctx.insert(
         "parents",
         &[
-            (index_url.as_str(), "top_level"),
-            ("./index.html", parent.as_str()),
+            (top_level_link, "top_level"),
+            (parent_link, parent.as_str()),
         ],
     );
     ctx.insert("stats", &stats);
-    ctx.insert("branch_enabled", &branch_enabled);
-    ctx.insert("precision", &precision);
 
     let mut file_buf = Vec::new();
     if let Err(e) = f.read_to_end(&mut file_buf) {
@@ -448,7 +492,7 @@ fn gen_html(
     let out = tera.render("file.html", &ctx).unwrap();
 
     if output.write_all(out.as_bytes()).is_err() {
-        eprintln!("Cannot write the file {:?}", output_file);
+        eprintln!("Cannot write the file {output_file:?}");
     }
 }
 
@@ -458,8 +502,7 @@ pub fn consumer_html(
     global: Arc<Mutex<HtmlGlobalStats>>,
     output: &Path,
     conf: Config,
-    branch_enabled: bool,
-    precision: usize,
+    abs_link_prefix: &Option<String>,
 ) {
     while let Ok(job) = receiver.recv() {
         if job.is_none() {
@@ -474,8 +517,7 @@ pub fn consumer_html(
             output,
             &job.rel_path,
             global.clone(),
-            branch_enabled,
-            precision,
+            abs_link_prefix,
         );
     }
 }
@@ -534,13 +576,13 @@ pub fn gen_badge(tera: &Tera, stats: &HtmlStats, conf: &Config, output: &Path, s
     create_parent(&output_file);
     let mut output_stream = match File::create(&output_file) {
         Err(_) => {
-            eprintln!("Cannot create file {:?}", output_file);
+            eprintln!("Cannot create file {output_file:?}");
             return;
         }
         Ok(f) => f,
     };
 
-    let mut ctx = make_context();
+    let mut ctx = make_context(conf, ".");
     ctx.insert(
         "current",
         &(get_percentage_of_covered_lines(stats.covered_lines, stats.total_lines) as usize),
@@ -551,7 +593,7 @@ pub fn gen_badge(tera: &Tera, stats: &HtmlStats, conf: &Config, output: &Path, s
     let out = tera.render(style.template_name(), &ctx).unwrap();
 
     if output_stream.write_all(out.as_bytes()).is_err() {
-        eprintln!("Cannot write the file {:?}", output_file);
+        eprintln!("Cannot write the file {output_file:?}");
     }
 }
 
@@ -582,7 +624,7 @@ pub fn gen_coverage_json(stats: &HtmlStats, conf: &Config, output: &Path, precis
     create_parent(&output_file);
     let mut output_stream = match File::create(&output_file) {
         Err(_) => {
-            eprintln!("Cannot create file {:?}", output_file);
+            eprintln!("Cannot create file {output_file:?}");
             return;
         }
         Ok(f) => f,
@@ -595,7 +637,7 @@ pub fn gen_coverage_json(stats: &HtmlStats, conf: &Config, output: &Path, precis
         &CoverageData {
             schema_version: 1,
             label: "coverage",
-            message: format!("{:.precision$}%", coverage),
+            message: format!("{coverage:.precision$}%"),
             color: if coverage >= conf.hi_limit {
                 "green"
             } else if coverage >= conf.med_limit {
@@ -607,7 +649,18 @@ pub fn gen_coverage_json(stats: &HtmlStats, conf: &Config, output: &Path, precis
     );
 
     if res.is_err() {
-        eprintln!("cannot write the file {:?}", output_file);
+        eprintln!("cannot write the file {output_file:?}");
+    }
+}
+
+pub fn gen_bundled_resources(conf: &Config, output: &Path) {
+    if !matches!(conf.html_resources, HtmlResources::Bundled) {
+        return;
+    }
+
+    let bulma = output.join("bulma.min.css");
+    if let Err(err) = std::fs::write(&bulma, BULMA_BUNDLED) {
+        eprintln!("cannot write {}: {err:?}", bulma.display());
     }
 }
 
