@@ -40,6 +40,9 @@ pub mod html;
 mod file_filter;
 pub use crate::file_filter::*;
 
+mod file_walker;
+pub use crate::file_walker::{find_binaries, ParallelWalker};
+
 use log::{error, warn};
 use std::fs;
 use std::io::{BufReader, Cursor};
@@ -175,6 +178,7 @@ pub fn consumer(
     branch_enabled: bool,
     guess_directory: bool,
     binary_path: Option<&Path>,
+    ignore_parsing_error: bool,
 ) {
     let mut gcov_type = GcovType::Unknown;
 
@@ -189,7 +193,7 @@ pub fn consumer(
                     ItemType::Path((stem, gcno_path)) => {
                         // GCC
                         if let Err(e) = run_gcov(&gcno_path, branch_enabled, working_dir) {
-                            error!("Error when running gcov: {}", e);
+                            error!("Error when running gcov: {e}");
                             continue;
                         };
                         let gcov_ext = get_gcov_output_ext();
@@ -260,7 +264,7 @@ pub fn consumer(
                             }
                             Err(e) => {
                                 // Just print the error, don't panic and continue
-                                error!("Error in computing counters: {}", e);
+                                error!("Error in computing counters: {e}");
                                 Vec::new()
                             }
                         }
@@ -275,14 +279,14 @@ pub fn consumer(
                     }
                 }
             }
-            ItemFormat::Profraw => {
+            ItemFormat::Profdata | ItemFormat::Profraw => {
                 if binary_path.is_none() {
                     error!("The path to the compiled binary must be given as an argument when source-based coverage is used");
                     continue;
                 }
 
                 if let ItemType::Paths(profraw_paths) = work_item.item {
-                    match llvm_tools::profraws_to_lcov(
+                    match llvm_tools::llvm_profiles_to_lcov(
                         profraw_paths.as_slice(),
                         binary_path.as_ref().unwrap(),
                         working_dir,
@@ -292,7 +296,7 @@ pub fn consumer(
 
                             for lcov in lcovs {
                                 new_results.append(&mut try_parse!(
-                                    parse_lcov(lcov, branch_enabled),
+                                    parse_lcov(lcov, branch_enabled, ignore_parsing_error),
                                     work_item.name
                                 ));
                             }
@@ -300,7 +304,7 @@ pub fn consumer(
                             new_results
                         }
                         Err(e) => {
-                            error!("Error while executing llvm tools: {}", e);
+                            error!("Error while executing llvm tools: {e}");
                             continue;
                         }
                     }
@@ -309,13 +313,19 @@ pub fn consumer(
                     continue;
                 }
             }
-            ItemFormat::Info | ItemFormat::JacocoXml => {
+            ItemFormat::Info | ItemFormat::JacocoXml | ItemFormat::Gocov => {
                 if let ItemType::Content(content) = work_item.item {
                     if work_item.format == ItemFormat::Info {
-                        try_parse!(parse_lcov(content, branch_enabled), work_item.name)
-                    } else {
+                        try_parse!(
+                            parse_lcov(content, branch_enabled, ignore_parsing_error),
+                            work_item.name
+                        )
+                    } else if work_item.format == ItemFormat::JacocoXml {
                         let buffer = BufReader::new(Cursor::new(content));
                         try_parse!(parse_jacoco_xml_report(buffer), work_item.name)
+                    } else {
+                        let mut buffer = BufReader::new(Cursor::new(content));
+                        try_parse!(parse_gocov(&mut buffer), work_item.name)
                     }
                 } else {
                     error!("Invalid content type");
@@ -432,7 +442,7 @@ mod tests {
             .expect("Failed to open lcov file");
         let mut buf = Vec::new();
         f.read_to_end(&mut buf).unwrap();
-        let results = parse_lcov(buf, false).unwrap();
+        let results = parse_lcov(buf, false, false).unwrap();
         let result_map: Arc<SyncCovResultMap> = Arc::new(Mutex::new(
             FxHashMap::with_capacity_and_hasher(1, Default::default()),
         ));
@@ -466,7 +476,7 @@ mod tests {
             .expect("Failed to open lcov file");
         let mut buf = Vec::new();
         f.read_to_end(&mut buf).unwrap();
-        let results = parse_lcov(buf, false).unwrap();
+        let results = parse_lcov(buf, false, false).unwrap();
         let result_map: Arc<SyncCovResultMap> = Arc::new(Mutex::new(
             FxHashMap::with_capacity_and_hasher(3, Default::default()),
         ));
