@@ -17,6 +17,7 @@ use std::str::FromStr;
 use std::sync::{Arc, Mutex};
 use std::{process, thread};
 
+use grcov::html::HtmlResources;
 use grcov::*;
 
 #[derive(Clone, PartialEq)]
@@ -48,7 +49,7 @@ impl FromStr for OutputType {
             "cobertura" => Self::Cobertura,
             "cobertura-pretty" => Self::CoberturaPretty,
             "markdown" => Self::Markdown,
-            _ => return Err(format!("{} is not a supported output type", s)),
+            _ => return Err(format!("{s} is not a supported output type")),
         })
     }
 }
@@ -90,7 +91,7 @@ impl FromStr for Filter {
         Ok(match s {
             "covered" => Self::Covered,
             "uncovered" => Self::Uncovered,
-            _ => return Err(format!("{} is not a supported filter", s)),
+            _ => return Err(format!("{s} is not a supported filter")),
         })
     }
 }
@@ -200,6 +201,9 @@ struct Opt {
     /// Ignore source files that can't be found on the disk.
     #[arg(long)]
     ignore_not_existing: bool,
+    /// Ignore error when parsing the coverage files.
+    #[arg(long)]
+    ignore_parsing_error: bool,
     /// Ignore files/directories specified as globs.
     #[arg(long = "ignore", value_name = "PATH", num_args = 1)]
     ignore_dir: Vec<String>,
@@ -274,6 +278,11 @@ struct Opt {
     /// Set the log level.
     #[arg(long, value_name = "LEVEL", default_value = "ERROR", value_enum)]
     log_level: LevelFilterArg,
+    /// Provides a json file with explicit list of files and lines to include into the report.
+    /// The json format is `{ "path": [[1, 2], ...], ... }`, where the numbers are inclusive
+    /// and 1-based line ranges.
+    #[arg(long, value_name = "PATH")]
+    incl_explicit: Option<PathBuf>,
     /// Lines in covered files containing this marker will be excluded.
     #[arg(long, value_name = "regex")]
     excl_line: Option<Regex>,
@@ -297,6 +306,15 @@ struct Opt {
     /// No symbol demangling.
     #[arg(long)]
     no_demangle: bool,
+    /// Generate absolute HTML links with the given prefix.
+    #[arg(long, value_name = "LINK PREFIX")]
+    abs_link_prefix: Option<String>,
+    /// Do not include the current date in the HTML report.
+    #[arg(long)]
+    no_date: bool,
+    /// Where to retrieve the CSS files used to style the HTML reports.
+    #[arg(long, value_name = "SOURCE", default_value = "bundled")]
+    html_resources: HtmlResources,
 }
 
 fn main() {
@@ -349,6 +367,7 @@ fn main() {
         opt.excl_br_line,
         opt.excl_br_start,
         opt.excl_br_stop,
+        opt.incl_explicit,
     );
     let demangle = !opt.no_demangle;
 
@@ -368,7 +387,7 @@ fn main() {
                 .copied()
                 .unwrap_or("<cause unknown>")
         });
-        error!("A panic occurred at {}:{}: {}", filename, line, cause);
+        error!("A panic occurred at {filename}:{line}: {cause}");
     }));
 
     let num_threads: usize = opt.threads.unwrap_or_else(|| 1.max(num_cpus::get() - 1));
@@ -426,14 +445,15 @@ fn main() {
     for i in 0..num_threads {
         let receiver = receiver.clone();
         let result_map = Arc::clone(&result_map);
-        let working_dir = tmp_path.join(format!("{}", i));
+        let working_dir = tmp_path.join(format!("{i}"));
         let source_root = source_root.clone();
         let binary_path = opt.binary_path.clone();
         let branch_enabled = opt.branch;
         let guess_directory = opt.guess_directory;
+        let ignore_parsing_error = opt.ignore_parsing_error;
 
         let t = thread::Builder::new()
-            .name(format!("Consumer {}", i))
+            .name(format!("Consumer {i}"))
             .spawn(move || {
                 fs::create_dir(&working_dir).expect("Failed to create working directory");
                 consumer(
@@ -444,6 +464,7 @@ fn main() {
                     branch_enabled,
                     guess_directory,
                     binary_path.as_deref(),
+                    ignore_parsing_error,
                 );
             })
             .unwrap();
@@ -562,6 +583,9 @@ fn main() {
                 opt.branch,
                 opt.output_config_file.as_deref(),
                 opt.precision,
+                &opt.abs_link_prefix.clone(),
+                opt.no_date,
+                opt.html_resources,
             ),
             OutputType::Cobertura => output_cobertura(
                 source_root.as_deref(),
