@@ -21,7 +21,7 @@ use tabled::{Table, Tabled};
 use uuid::Uuid;
 
 use crate::defs::*;
-use crate::html;
+use crate::html::{self, HtmlResources};
 
 macro_rules! demangle {
     ($name: expr, $demangle: expr, $options: expr) => {{
@@ -296,11 +296,11 @@ pub fn output_lcov(results: &[ResultTuple], output_file: Option<&Path>, demangle
             }
         }
 
-        writeln!(writer, "BRF:{}", branch_count).unwrap();
-        writeln!(writer, "BRH:{}", branch_hit).unwrap();
+        writeln!(writer, "BRF:{branch_count}").unwrap();
+        writeln!(writer, "BRH:{branch_hit}").unwrap();
 
         for (line, execution_count) in &result.lines {
-            writeln!(writer, "DA:{},{}", line, execution_count).unwrap();
+            writeln!(writer, "DA:{line},{execution_count}").unwrap();
         }
         writeln!(writer, "LF:{}", result.lines.len()).unwrap();
         writeln!(
@@ -374,7 +374,7 @@ fn get_coveralls_git_info(commit_sha: &str, vcs_branch: &str) -> Value {
         get_git_output([
             "log",
             "--max-count=1",
-            &format!("--pretty=format:{}", format),
+            &format!("--pretty=format:{format}"),
             commit_sha,
         ])
     };
@@ -526,6 +526,9 @@ pub fn output_html(
     branch_enabled: bool,
     output_config_file: Option<&Path>,
     precision: usize,
+    abs_link_prefix: &Option<String>,
+    no_date: bool,
+    html_resources: HtmlResources,
 ) {
     let output = if let Some(output_dir) = output_dir {
         PathBuf::from(output_dir)
@@ -545,17 +548,29 @@ pub fn output_html(
 
     let (sender, receiver) = unbounded();
 
-    let stats = Arc::new(Mutex::new(HtmlGlobalStats::default()));
+    let global_stats = HtmlGlobalStats {
+        abs_prefix: abs_link_prefix.clone().map(PathBuf::from),
+        ..Default::default()
+    };
+    let stats = Arc::new(Mutex::new(global_stats));
     let mut threads = Vec::with_capacity(num_threads);
-    let (tera, config) = html::get_config(output_config_file);
+    let (tera, config) = html::get_config(
+        output_config_file,
+        branch_enabled,
+        precision,
+        no_date,
+        html_resources,
+    );
+    let abs_link_prefix_owned = abs_link_prefix.clone();
     for i in 0..num_threads {
         let receiver = receiver.clone();
         let output = output.clone();
         let config = config.clone();
         let stats = stats.clone();
         let tera = tera.clone();
+        let abs_link_prefix_thread = abs_link_prefix_owned.clone();
         let t = thread::Builder::new()
-            .name(format!("Consumer HTML {}", i))
+            .name(format!("Consumer HTML {i}"))
             .spawn(move || {
                 html::consumer_html(
                     &tera,
@@ -563,8 +578,7 @@ pub fn output_html(
                     stats,
                     &output,
                     config,
-                    branch_enabled,
-                    precision,
+                    &abs_link_prefix_thread,
                 );
             })
             .unwrap();
@@ -594,13 +608,14 @@ pub fn output_html(
 
     let global = Arc::try_unwrap(stats).unwrap().into_inner().unwrap();
 
-    html::gen_index(&tera, &global, &config, &output, branch_enabled, precision);
+    html::gen_index(&tera, &global, &config, &output);
 
     for style in html::BadgeStyle::iter() {
         html::gen_badge(&tera, &global.stats, &config, &output, style);
     }
 
     html::gen_coverage_json(&global.stats, &config, &output, precision);
+    html::gen_bundled_resources(&config, &output);
 }
 
 pub fn output_markdown(results: &[ResultTuple], output_file: Option<&Path>, precision: usize) {
@@ -616,7 +631,7 @@ pub fn output_markdown(results: &[ResultTuple], output_file: Option<&Path>, prec
         if start == end {
             start.to_string()
         } else {
-            format!("{}-{}", start, end)
+            format!("{start}-{end}")
         }
     }
 
